#include <Servo.h>
#include <Wire.h> 
#include <LiquidCrystal_I2C.h>

LiquidCrystal_I2C lcd(0x27,16,2);   // Задаем адрес и размерность дисплея. 
Servo esc;

#define THROTTLE_SIGNAL_IN 0 // INTERRUPT 0 = DIGITAL PIN 2 - use the interrupt number in attachInterrupt
#define THROTTLE_SIGNAL_IN_PIN 2 // INTERRUPT 0 = DIGITAL PIN 2 - use the PIN number in digitalRead
#define NEUTRAL_THROTTLE 1500 // this is the duration in microseconds of neutral throttle on an electric RC Car

int throttlePin = 0;
int stopPin = 8;
int escPIN = 11;

volatile float time = 0;
volatile float time_last = 0;
volatile long rt_blades = 0;
volatile static unsigned long lastPressed=0;
volatile boolean ledOn = false;
volatile int rpm_array[5] = {0,0,0,0,0};
volatile int blades = 2;

int pwm = 0;
int pwm_last = 0;
float time_step = 0;
int next = 0;
long rotations = 0;
long rpm = 0;
float time_current = 0;
boolean gogogo = false;
boolean stoping = false;


//=====================================
void setup() {
  Serial.begin(9600); 
  Serial.println("TACHOMETER 3");
  pinMode(13, OUTPUT);
  pinMode(throttlePin, INPUT);
  pinMode(stopPin, INPUT);
  esc.attach(escPIN,1000,2000);
  attachInterrupt(1, fan_interrupt, FALLING);
  attachInterrupt(0, swap, RISING);
  
  lcd.init();                
  lcd.backlight();           
  lcd.print("  TACHOMETER 3"); 
  delay(500);
  lcd.clear();
}
//=====================================
void fan_interrupt()
{
  time_current = micros();
  time = (time_current - time_last); 
  time_last = time_current;
  rt_blades++;
}
//=====================================
void swap(){
  if(millis()-lastPressed<300)return;// если с прошлого нажатия прошло меньше 100ms - игнорируем этот импульс
  lastPressed=millis();// запомнили когда было последние нажатие
  ledOn=!ledOn;
  digitalWrite(13,ledOn);
  blades++;
  if (blades>9) blades=1;
  Serial.print("Button Pressed, blases: ");
  Serial.println(blades);
}
//=====================================
void loop() {
  
  int throttle = analogRead(throttlePin);
  if (!stoping) {
    pwm = map(throttle, 0, 1023, 2000, 1000);
    throttle = map(throttle, 0, 1023, 179, 0);
    esc.write(throttle);
  }
  
//----  
  if (micros()-time_step > 400000) {
    gogogo = true;
    time_step = micros();
    
   // кнопка стопкран
    if (digitalRead(stopPin) == LOW) {
      stoping = !stoping;
      if (stoping) { 
        lcd.setCursor(8, 0); lcd.print("-"); Serial.println("-");
        pwm_last = pwm;
        StartStoping(pwm_last);
      } else {
        lcd.setCursor(8, 0); lcd.print("+"); Serial.println("+");
        EndStoping(pwm_last);
      }  
    }  

  }  
//----  
  if ((time > 0)&(gogogo)) {
    rpm_array[next] = 60*(1000000/(time*blades));
    next++;
    if (next > 4) next=0;
    rpm = 0;
    for (int i=0; i<=4; i++) rpm=rpm+rpm_array[i];
    rpm = rpm / 5;

    for (int i=0; i<=4; i++) {
      Serial.print(rpm_array[i]);
      Serial.print(" ");
    }  
    Serial.println();
    
    lcd_print();
    gogogo=false;
    time = 0;      
//----    
  } else if ((time < 1)&(gogogo)) {  /// нет вращения - забиваем нулями
    rpm_array[next] = 0;
    next++;
    if (next > 4) next=0;
    rpm = 0;
    for (int i=0; i<=4; i++) rpm=rpm+rpm_array[i];
    rpm = rpm / 5;
    for (int i=0; i<=4; i++) {
      Serial.print(rpm_array[i]);
      Serial.print(" ");
    }  
    Serial.println(); 
    lcd_print();
    gogogo=false;
    
  }  
 
}
//=====================================
void lcd_print(){
  
//  lcd.clear();
  if (!stoping) { 
    lcd.setCursor(0, 0);
    lcd.print("PWM:");
    lcd.print(pwm);
  }  
  lcd.setCursor(10, 0);
  lcd.print("BLD:");
  lcd.print(blades);
  lcd.setCursor(0, 1);    
  lcd.print("RPM:");      
  lcd.print(rpm);
  lcd.print("     ");      
  lcd.setCursor(10, 1);    
  lcd.print("R:    ");   
  lcd.setCursor(12, 1);    
  if (rt_blades/blades>=1000) rt_blades=0;
  rotations = rt_blades/blades;
  lcd.print(rotations);
}

//=======================================
void StartStoping(int pwm_cur) {

  int pwm_down = (pwm_cur - 1000) / 10;  
  int pwm_now = pwm_cur;

   for (int i=0; i <= 9; i++) {
     pwm_now = pwm_now - pwm_down;
     esc.write(pwm_now);
     lcd.setCursor(0, 0);
     lcd.print("PWM:");
     lcd.print(pwm_now);
     Serial.print(pwm_now); Serial.print(" ");
     delay(200);
   }  
}
//=======================================
void EndStoping(int pwm_cur) {

  int pwm_up = (pwm_cur - 1000) / 10;  
  int pwm_now = 1000;

   for (int i=0; i <= 9; i++) {
     pwm_now = pwm_now + pwm_up;
     esc.write(pwm_now);
     lcd.setCursor(0, 0);
     lcd.print("PWM:");
     lcd.print(pwm_now);
     Serial.print(pwm_now); Serial.print(" ");
     delay(200);
   }  

}


